function WeatherData() {
}

WeatherData.prototype.weekDayNames = ["Sun","Mon","Tue","Wed","Thu","Fri","Sat"];
WeatherData.prototype.monthNames   = ["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];

WeatherData.prototype.temperatureUnit;
WeatherData.prototype.rainfallUnit;
WeatherData.prototype.pressureUnit
WeatherData.prototype.windspeedUnit;
WeatherData.prototype.showHumidity;
WeatherData.prototype.showPressure;
WeatherData.prototype.showRain;

WeatherData.prototype.clientRawArray=null;
WeatherData.prototype.clientRawExtraArray=null;

WeatherData.prototype.setClientRaw = function(data)
{
    this.clientRawExtraArray = null;
    
    if (data.match(/^12345/))
    {
        // clean up any carriage returns
        data = data.replace(/\r|\n|\r\n/g,"");
        
        //replace multiple spaces!
        data = data.replace("      "," ");
        data = data.replace("  "," ");
        
        this.clientRawArray = data.split(" ");  
        return true;
     }
    else
      return false;
      
}

WeatherData.prototype.setClientRawExtra = function(data)
{
    if (data.match(/^12345/))
    {
        // clean up any carriage returns
        data = data.replace(/\r|\n|\r\n/g,"");
        
        //replace multiple spaces!
        data = data.replace("      "," ");
        data = data.replace("  "," ");
        
        this.clientRawExtraArray = data.split(" ");  
        return true;
     }
    else
      return false;
}

WeatherData.prototype.getDateTime = function()
{
    var day = parseFloat(this.clientRawArray[35]);
    var month = parseFloat(this.clientRawArray[36]);
    var year = parseFloat(this.clientRawArray[141]);
    if (isNaN(year))
       year = new Date().getYear();
               
    var hour = parseFloat(this.clientRawArray[29]);
    var min = parseFloat(this.clientRawArray[30]);
    var sec = parseFloat(this.clientRawArray[31]);
    return new Date(year, month-1, day, hour, min, sec);
}

WeatherData.prototype.getAgeOfClientRawInHours = function()
{
    var d = this.getDateTime();
    var one_hour=1000*60*60;

    return (Math.ceil(new Date().getTime() - d.getTime()) / one_hour).toFixed(0);
}

WeatherData.prototype.getForecast = function()
{
    var forecast = this.clientRawArray[49].replace(/_/g," ").replace(/\\/g,", ");
    if (this.clientRawExtraArray[531].length > 0)
        forecast += ", "+this.clientRawExtraArray[531].replace(/_/g," ");
    return forecast;
}

WeatherData.prototype.formatVersions = function(gadgetVersion)
{
    var versions = "";
    
    if ((this.clientRawArray != null) && (this.clientRawArray.length > 0))
        versions += this.clientRawArray[this.clientRawArray.length-2].replace(/!/g,"")+" - ";
    if (this.clientRawExtraArray != null)
        versions += this.clientRawExtraArray[this.clientRawExtraArray.length-2].replace(/!/g,"")+ " - ";
    versions += "G"+gadgetVersion; 
        
    return versions;
}

WeatherData.prototype.formatUV = function(value)
{
    value = parseFloat(value);
    var condition = "";
    if (value < 2)
      condition = "low";
    else if (value < 5)
      condition = "moderate";
    else if (value < 7)
      condition = "high";
    else if (value < 9)
      condition = "very high";
    else
      condition = "extreme";
    return value+" - " + condition;
}

WeatherData.prototype.getTrendImage = function(PanelIndex)
{
  if (this.clientRawArray == null)
    return "";
  else if (PanelIndex == 0)  // Temperature
  {
      var tempTrend = parseFloat(this.clientRawArray[143]);
      return (tempTrend < 0) ? "falling" : (tempTrend > 0) ? "rising" : "";
  }
  else if (PanelIndex == 1) // Humdidity
  {
      var humidityTrend = parseFloat(this.clientRawArray[144]);
      return (humidityTrend < 0) ? "falling" : (humidityTrend > 0) ? "rising" : "";
  }
  else if (PanelIndex == 2) // Pressure
  {
      var baroTrend = parseFloat(this.clientRawArray[50]);
      return (baroTrend < 0) ? "falling" : (baroTrend > 0) ? "rising" : "";
  }
  else
  {
      return "";
  }
}

WeatherData.prototype.formatSun = function(value)
{
    value = parseFloat(value);
    if (isNaN(value))
        return "N/A";
    else if (value == 1)
        return "1 hour";
    else
        return value.toFixed(2) + " hours";    
 }


WeatherData.prototype.formatRainSymbol = function()
{
    return (this.rainfallUnit == 0) ? "mm" : "\"";
}

WeatherData.prototype.formatRain = function(value)
{
    var tmp = parseFloat(value);
    if (this.rainfallUnit== 1) // Convert to Inches
        tmp = Math.round( (tmp / 25.4) * 100) / 100;
    return tmp.toFixed(2);
}

WeatherData.prototype.formatDateTime = function()
{
    var d = this.getDateTime();
    var hour = d.getHours();
    var min = d.getMinutes();
    var sec = d.getSeconds();
    return this.weekDayNames[d.getDay()]+" "+d.getDate()+" "+this.monthNames[d.getMonth()]+" "+d.getFullYear() +"  "+ 
        ((hour > 9) ? hour : "0" + hour)+":"+
        ((min > 9) ? min : "0" + min)+":"+
        ((sec > 9) ? sec : "0" + sec);
}

WeatherData.prototype.formatHumidity = function(value)
{
    return value+"%";
}

WeatherData.prototype.formatWindSpeed = function(value)
{
    var tmp = parseFloat(value);
    if (this.windspeedUnit == 0)
        return (tmp * 1.852).toFixed(1)+" km/h";
    else if (this.windspeedUnit == 1)
    {        
        return (tmp * 1.15).toFixed(1)+" mph";
    }
    else if (this.windspeedUnit == 2)
    {
        return tmp.toFixed(1)+" kts";
    }   
    else
    {
        return (tmp * 0.514444).toFixed(1) + " m/s"; // Meters / Second : http://www.johnsnhweather.com/formulas/windConversion.html
    }
}

WeatherData.prototype.formatPressure = function(value)
{
    // HPA
    var tmp = parseFloat(value);
    
    if (this.pressureUnit == 0) // hPa
        return tmp.toFixed(1);
    else if (this.pressureUnit == 1) // Inches
        {
            tmp = Math.round(tmp*0.014503773773*2.036021*1000000)/1000000;
            return tmp.toFixed(2);
            
        }
    else  // millibars
        return tmp.toFixed(1); // Math.round(tmp*0.014503773773*0.014503773773*1000000)/1000000
        
    // http://www.cecomp.com/pressure_conversion.htm    
        
}


WeatherData.prototype.formatTemperature = function(value)
{
    var tmp = parseFloat(value);
    if (this.temperatureUnit == 1) // Convert to Fahrenheit
        tmp = (tmp * 1.8) + 32;
    return tmp.toFixed(1) + "&#176;";
}

/********************************************************************************/

WeatherData.prototype.temperatureCurrent = function()
{
    return parseFloat(this.clientRawArray[4]);
}

WeatherData.prototype.humidityCurrent = function()
{
    return parseInt(this.clientRawArray[5]);
}

WeatherData.prototype.pressureCurrent = function()
{
    return parseFloat(this.clientRawArray[6]);
}

WeatherData.prototype.windspeedCurrent = function()
{
    return parseFloat(this.clientRawArray[1]);
}

WeatherData.prototype.windDirectionCurrent = function()
{
    return parseFloat(this.clientRawArray[3]);
}


WeatherData.prototype.rainToday = function()
{
    return parseFloat(this.clientRawArray[7]);
}



WeatherData.prototype.weatherConditions = function()
{
    return this.clientRawArray[49].replace(/_/g," ").toLowerCase();
}
       
WeatherData.prototype.getBackground = function()
{
  var temperature = this.temperatureCurrent();
  var conditions  = this.weatherConditions();
  var Background;
  
  if (System.Gadget.Settings.read("OptionsImages") == 0)
    Background = "images/background.png";
  else if (conditions.match("night") != null)
    {
        if (temperature < 5)
            Background = "images/background_night_cold.png";
        else if (temperature < 18)
            Background = "images/background_night_temperate.png";
        else
            Background = "images/background_night_hot.png";
    }
    else if (conditions.match("dawn") != null)
    {
        if ((conditions.match("rain") != null) || (conditions.match("drizzle") != null))
            Background = "images/background_dawn_rain.png";
        else if (conditions.match("cloud") != null)
            Background = "images/background_dawn_cloudy.png";		
        else
            Background = "images/background_dawn_temperate.png";		    
    }
    else if (conditions.match("dusk") != null)
    {
        if ((conditions.match("rain") != null) || (conditions.match("drizzle") != null))
            Background = "images/background_dusk_rain.png";
        else if (conditions.match("cloud") != null)
            Background = "images/background_dusk_cloudy.png";		
        else
            Background = "images/background_dusk_sunny.png";		    
    }
    else 
    {
        if (((conditions.match("rain") != null) || (conditions.match("drizzle") != null)) && (conditions.match("stopped") == null))
        {
            if (temperature < 5)
                Background = "images/background_day_rain_below5.png";
            else if (temperature < 15)
                Background = "images/background_day_rain_below15.png";
            else if (temperature < 25)
                Background = "images/background_day_rain_below25.png";
            else
                Background = "images/background_day_rain_hot.png";
        }   
        else if (conditions.match("cloud") != null)
        {
            if (temperature < -5)
                Background = "images/background_day_cloudy_belowminus5.png";		    
            else if (temperature < 0)
                Background = "images/background_day_cloudy_below0.png";		    
            else if (temperature < 5)
                Background = "images/background_day_cloudy_below5.png";		    
            else if (temperature < 15)
                Background = "images/background_day_cloudy_below15.png";		    
            else if (temperature < 25)
                Background = "images/background_day_cloudy_below25.png";		    
            else if (temperature < 35)
                Background = "images/background_day_cloudy_below35.png";		    
            else 
                Background = "images/background_day_cloudy_hot.png";	
        }	
        else
        {
            if (temperature < -5)
                Background = "images/background_day_sunny_belowminus5.png";		    
            else if (temperature < 0)
                Background = "images/background_day_sunny_below0.png";		    
            else if (temperature < 5)
                Background = "images/background_day_sunny_below5.png";		    
            else if (temperature < 15)
                Background = "images/background_day_sunny_below15.png";		    
            else if (temperature < 25)
                Background = "images/background_day_sunny_below25.png";		    
            else if (temperature < 35)
                Background = "images/background_day_sunny_below35.png";		    
            else 
                Background = "images/background_day_sunny_hot.png";		    
        }
    }
    
    return Background;
}            		            		
  
    
WeatherData.prototype.station = function()
{
    var stationID = this.clientRawArray[32].replace(/_/g,"");
    return stationID.substring(0,stationID.indexOf('-'));
}

WeatherData.prototype.stationName = function()
{
    return System.Gadget.Settings.read("WeatherName");
}

WeatherData.prototype.getTempTrendData = function()
{
    var results = new Array(24);
    var temps = new Array(24);
    for (i=0; i<=temps.length;i++)
       temps[0] = 0;
    
    for (i=21; i<=40;i++)
        temps[i-21] = this.clientRawExtraArray[i];
    for (i=566; i<=569;i++)
        if (i < this.clientRawExtraArray.length)
            temps[i-566+20] = this.clientRawExtraArray[i];
        else
            temps[i-566+20] = -99999;
        
    // Determine the current hour...
    var Hour = parseFloat(this.clientRawArray[29]);
    for (i = 0; i <= 23;i++)
    {   
        var hourStr = (Hour == 23) ? "12<br>am" : (Hour == 11) ? "12<br>pm" : (Hour > 11) ? Hour-11 : Hour+1;
        if (i==0)
          hourStr += (Hour > 11) ? "<br>pm" : " <br>am";
        results[i] = new Array(hourStr, this.formatTemperature(temps[i]));
        Hour++;
        if (Hour == 24)
            Hour = 0;
    }            
    return results;
}

WeatherData.prototype.getPressureTrendData = function()
{
    var results = new Array(24);
    var temps = new Array(24);
    
    for (i=439; i<=458;i++)
        temps[i-439] = this.clientRawExtraArray[i];
    for (i=574; i<=577;i++)
        if (i < this.clientRawExtraArray.length)
            temps[i-574+20] = this.clientRawExtraArray[i];
        else
            temps[i-574+20] = -99999;
        
    // Determine the current hour...
    var Hour = parseFloat(this.clientRawArray[29]);
    for (i = 0; i<=23;i++)
    {
        var hourStr = (Hour == 23) ? "12<br>am" : (Hour == 11) ? "12<br>pm" : (Hour > 11) ? Hour-11 : Hour+1;
        if (i==0)
          hourStr += (Hour > 11) ? "<br>pm" : " <br>am";
        results[i] = new Array(hourStr, this.formatPressure(temps[i]));
        Hour++;
        if (Hour == 24)
            Hour = 0;
    }            
    return results;
}
 
WeatherData.prototype.getRainYesterday = function()
{
    // I have found that clientRaw field 019 Yesterday Rain - does not actually reflect yesterday all of the time.
    
    var d = this.getDateTime();
    d.setDate(d.getDate()-1);

    // get the day of week
    var dayOfWeek = d.getDay();
    // dayOfWeek = 0 = Sunday, 1 = Monday etc...
    return (dayOfWeek == 0) ? (parseFloat(this.clientRawExtraArray[490]) / 10) : (parseFloat(this.clientRawExtraArray[484+dayOfWeek-1]) / 10);
} 

WeatherData.prototype.getRainTrendData = function()
{
    var d = this.getDateTime();
    var results = new Array(7);
    var temps = new Array(24);
                   
    // 484 = Monday, 490 = Sunday               
    
    temps[0] = parseFloat(this.clientRawExtraArray[490]) / 10;  
    for (i=484; i<=489;i++)
       // Is this a Weather Display bug? Rainfall seems to be 10 times the actual amount that
       // is stored in the Clientrawdaily.txt
       temps[1+(i-484)] = parseFloat(this.clientRawExtraArray[i]) / 10;  
    
    d.setDate(d.getDate()-6);
    for (i = 0; i<=6;i++)
    {
        if (i == 6)
            results[i] = new Array(this.weekDayNames[d.getDay()], this.formatRain(this.rainToday()));
        else
            results[i] = new Array(this.weekDayNames[d.getDay()], this.formatRain(temps[d.getDay()]));
        d.setDate(d.getDate()+1);
    }            
    return results;
}